﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "Weapon/LMWeaponManager.h"

#include "Weapon/LMWeaponBase.h"


ULMWeaponManager::ULMWeaponManager()
{
	PrimaryComponentTick.bCanEverTick = false;
}

void ULMWeaponManager::BeginPlay()
{
	Super::BeginPlay();
	
	if (AmmoData.IsEmpty())
		SetupAmmoData();
}

void ULMWeaponManager::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent)
{
	Super::PostEditChangeProperty(PropertyChangedEvent);
	
	if (AmmoDataTable)
		SetupAmmoData();
}

void ULMWeaponManager::SetupAmmoData()
{
	TArray<FLMAmmoData*> Rows;
	AmmoData.Empty();
	
	AmmoDataTable->GetAllRows(TEXT(""), Rows);
	for (const auto Data : Rows)
	{
		if (AmmoData.Contains(Data->AmmoType))
			continue;
		
		AmmoData.Add(Data->AmmoType, *Data);
	}
}

int ULMWeaponManager::RemoveAmmo(const EAmmoType Ammo, const int Count)
{
	if (AmmoData.Contains(Ammo))
	{
		FLMAmmoData &Data = AmmoData[Ammo];

		int Difference = Data.AmmoCount - Count;
		UE_LOG(LogTemp, Display, TEXT("Difference:  %i | Removing: %i | Storage: %i"), Difference, Count, Data.AmmoCount)
		if (Difference < 0)
		{
			UE_LOG(LogTemp, Error, TEXT("%i - %i = %i"), Data.AmmoCount, Count, Data.AmmoCount - Count);
			Data.AmmoCount = FMath::Clamp(Data.AmmoCount - Count, 0, Data.MaxAmmo);
			UE_LOG(LogTemp, Warning, TEXT("Difference < 0 | Munition to add: %i | Removed: %i | New Munition: %i"), Count + Difference, Count, Data.AmmoCount)
			return Count + Difference;
		}
		
		Data.AmmoCount -= Count;
		UE_LOG(LogTemp, Warning, TEXT("New munition: %i | Removed: %i"), Data.AmmoCount, Count)
		return Count;
	}

	return 0;
}

void ULMWeaponManager::Initialize(USkeletalMeshComponent* Mesh)
{
	ArmsMesh = Mesh;
	
	for (auto WeaponTemplate : StartingWeapons)
	{
		if (!WeaponTemplate)
			continue;
		
		ALMWeaponBase* Instance = GetWorld()->SpawnActor<ALMWeaponBase>(WeaponTemplate);
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 2.f, FColor::Blue, TEXT("Spawing"));
		Instance->Setup(ArmsMesh, GetOwner());
		Weapons.Add(Instance);
	}
	
	if (!Weapons.IsEmpty())
	{
		SetWeapon(CurrentWeaponIndex);
	}
}

void ULMWeaponManager::AddAmmoType(EAmmoType AmmoType, int AmmoCount)
{
	FString Debug = FString::Printf(TEXT("Adding %i ammo of type %i"), AmmoCount, AmmoType);
	GEngine->AddOnScreenDebugMessage(1, 1.f, FColor::Cyan, Debug);

	if (AmmoData.Contains(AmmoType))
	{
		AmmoData[AmmoType].AddAmmo(AmmoCount);
	}
	else
	{
		const FString Warning = FString::Printf(TEXT("Ammo type %i not found"), AmmoType);
		GEngine->AddOnScreenDebugMessage(1, 2.f, FColor::Orange, Warning);
		
		FLMAmmoData Data = FLMAmmoData();
		Data.AmmoType = AmmoType;
		Data.AmmoCount = AmmoCount;
		AmmoData[AmmoType] = Data;
	}
}

int ULMWeaponManager::GetAmmoCount(const EAmmoType AmmoType)
{
	if (AmmoData.Contains(AmmoType))
		return AmmoData[AmmoType].AmmoCount;

	return 0;
}

void ULMWeaponManager::Fire()
{
	ALMWeaponBase* Weapon = GetCurrentWeapon();
	Weapon->PrimaryFire();
}

void ULMWeaponManager::Reload()
{
	GEngine->AddOnScreenDebugMessage(3, 1.f, FColor::Cyan, "Reloading");
	
	ALMWeaponBase* Weapon = GetCurrentWeapon();
	Weapon->Reload();
}

void ULMWeaponManager::SwitchWeapon(const int Direction)
{
	const int NewIndex = Direction + CurrentWeaponIndex;
	if (NewIndex >= 0 && NewIndex < Weapons.Num())
	{
		SetWeapon(NewIndex);
	}
	else
	{
		if (NewIndex < 0)
			SetWeapon(Weapons.Num() - 1);
		else
			SetWeapon(0);
	}
}

void ULMWeaponManager::SetWeapon(const int Index)
{
	if (Index < 0 || Index >= Weapons.Num())
	{
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 5, FColor::Red, "Invalid Weapon Index");
		return;
	}

	GetCurrentWeapon()->SetActorHiddenInGame(true);
	CurrentWeaponIndex = Index;
	GetCurrentWeapon()->SetActorHiddenInGame(false);
}

